#!/usr/bin/python
# -*- coding: utf-8 -*- 

#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#    Dieses Programm ist Freie Software: Sie können es unter den Bedingungen
#    der GNU General Public License, wie von der Free Software Foundation,
#    Version 3 der Lizenz oder (nach Ihrer Wahl) jeder neueren
#    veröffentlichten Version, weiterverbreiten und/oder modifizieren.
#
#    Dieses Programm wird in der Hoffnung, dass es nützlich sein wird, aber
#    OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
#    Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
#    Siehe die GNU General Public License für weitere Details.
#
#    Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
#    Programm erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.

# see README.md for an idea what ax88179_178a-pinger does

import ping
import plac
import time
import subprocess as sp
import logging
import signal

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)
ch = logging.StreamHandler()
ch.setLevel(logging.INFO)
logger.addHandler(ch)

rmmod_default = "rmmod"
modprobe_default = "modprobe"
check_interval_default=15
timeout_default=5
ax88179_178a = "ax88179_178a"
check_interval_interruptible = 0.1

class Ax88179_178aPinger():
    def __init__(self, iface, dest_addr, check_interval=15, timeout=5, 
    rmmod=rmmod_default, modprobe=modprobe_default):
        self.running = True
        self.iface = iface
        self.dest_addr = dest_addr
        if check_interval < check_interval_interruptible:
            raise ValueError("check_interval %d mustn't be smaller than %d" % (check_interval, check_interval_interruptible, ))
        self.check_interval = check_interval
        self.timeout = timeout
        self.rmmod = rmmod
        self.modprobe = modprobe
    
    def ping_loop(self):
        while(self.running):
            logger.debug("pinging host '%s'" % (self.dest_addr, ))
            ping_tuple = ping.quiet_ping(self.dest_addr, self.timeout, count=3, 
                psize=64
            )
            ping_percentage = ping_tuple[0]
            logger.debug("got %d percent package loss during ping" % 
                (ping_percentage, )
            )
            if ping_percentage == 100:
                # everything above 0 is considered high latency, but the network 
                # still working
                logger.info("""detected 100 percent package loss, un- and reloading 
                    driver '%s'""" % (ax88179_178a, ))
                sp.check_call([self.rmmod, ax88179_178a])
                sp.check_call([self.modprobe, ax88179_178a])
            logger.debug("waiting for check interval of %d seconds to elapse" % 
                (self.check_interval, )
            )
            # in order to be able to respond to SIGTERM without sleeping the whole 
            # `check_interval`, we sleep `check_interval/
            # check_interval_interruptible*check_interval_interruptible`
            check_interval_elapsed = 0
            while check_interval_elapsed < self.check_interval and self.running:
                time.sleep(check_interval_interruptible)
                check_interval_elapsed += check_interval_interruptible
    
    def isRunning(self):
        return self.running
    
    def setRunning(self, running):
        self.running = running

@plac.annotations(
    iface=("The interface to watch over for driver failures"), 
    dest_addr=("The destination address for the ping"), 
    check_interval=("""The interval in seconds between checks. Checks occur 
        between every `check_interval` up to `check_interval+timeout` seconds 
        depending on how high the network latency is. If you set this value too 
        low the ping might fail due to network latency instead of a driver 
        failure and unnecessarily un- and reload the driver (that might even 
        increase network latency and get the script stuck in un- and reloading 
        every check interval. If you set it too high, your services might fail 
        because their request time out before the connection could be 
        reestablished after un- and reloading the driver. The latter is then 
        missing its only purpose.""", "option"), 
    timeout=("""The timeout for the ping in seconds. See the description of 
        `check_interval` for warning about value boundaries which involve and 
        apply to `timeout` as well.""", "option"), 
    rmmod=("The rmmod binary to use", "option"), 
    modprobe=("The modprobe binary to use", "option"), 
    debug=("Enable detailed output of application status", "flag"), 
)
def ax88179_178a_pinger(iface, dest_addr, check_interval=check_interval_default, timeout=timeout_default, 
    rmmod=rmmod_default, modprobe=modprobe_default, debug=False
):
    if debug:
        logger.setLevel(logging.DEBUG)
        ch.setLevel(logging.DEBUG)
    ax88179_178a_pinger = Ax88179_178aPinger(iface=iface, dest_addr=dest_addr, check_interval=check_interval, timeout=timeout, 
    rmmod=rmmod, modprobe=modprobe)
    def handler(signum, frame):
        logger.debug("signal handler called with signal %d" % (signum, ))
        if ax88179_178a_pinger.isRunning():
            ax88179_178a_pinger.setRunning(False)
            logger.info("Eventullay waiting for ping  command to return. Send signal SIGTERM again to force interrupt.")
        else:
            raise Exception("Interruption forced with second signal SIGTERM")
    signal.signal(signal.SIGINT, handler)
    ax88179_178a_pinger.ping_loop()

if __name__ == "__main__":
    plac.call(ax88179_178a_pinger)
